# for adapters
FRS_REMOVED_FOLDER_NAME = '.frsremoved'
FRS_ARCHIVED_FOLDER_NAME = 'archived'
FRS_CACHE_FOLDER_PREFIX = '.frs.'

import sys
# FS_CHARSET = 'gbk'
FS_CHARSET = sys.getfilesystemencoding()
